function [y,c,k,m,A,x,nr,r,pit,pite,ferror1,ferror2]=S_M_sr(eps,X0,XSS,nobs,param)

% [Copyright J. Ruiz (2006)]

% structural parameters

alpha=param(1);     % Output elasticity of capital stock
delta=param(2);     % Depreciation rate of capital
theta=param(3);     % weight of the real money in utility
sigma=param(4);     % Inverse of intertemporal elasticity of consumption
beta=param(5);      % Time discount factor
tauy=param(6);      % Income tax rate
tauc=param(7);      % Consumption tax rate
xss=param(8);       % growth rate of money supply
rhox=param(9);      % autoregressive parameter for the growth rate of money supply
Ass=param(10);      % steady state for technology variable
rhoA=param(11);     % Autoregressive parameter for the technology variable

% Steady state

kss=XSS(1);
css=XSS(2);
rss=XSS(3);
piss=XSS(4);
iss=XSS(5);
mss=XSS(6);
yss=XSS(7);

% initial conditions

k0=X0(1);
b0=X0(2);
M0=X0(3);
x0=X0(4);
A0=X0(5);

% Computing Matrix B D F Gamma1 Gamma2

MB=[0           0                          kss;
    sigma/beta  -theta*(1-sigma)/beta      (1-alpha)*((1/beta)-(1-delta));
    sigma/beta  -(theta*(1-sigma)+1)/beta  0];

MD=[-css        0                               Ass*alpha*(kss^alpha)+(1-delta)*kss;
    sigma/beta  -theta*(1-sigma)/beta           0;
    sigma/beta  -(theta*(1-sigma)+1+iss)/beta   0];

MF=[Ass*(kss^alpha)             0;
    ((1/beta)-(1-delta))*rhoA   0;
    0                           -1/beta];

MGamma1=inv(MB)*MD;
MGamma2=inv(MB)*MF;
rhos=[rhoA 0; 0 rhox];
MGamma3=[MGamma1 MGamma2; zeros(2,3)  rhos];
MGamma4=[1 0; 0 1; 0 0; 0 0; 0 0];
MGamma5=[0 0; 0 0; 0 0; 1 0; 0 1];

% Computing matrix M^(-1)

% Since M is the matrix having as columns the right eigenvectors 
% of MGamma3, its inverse has as rows the left eigenvectors of MGamma3

[evec,eval]=eig(MGamma3');
    
    ZI=[];
    ZR=[];
    AVALR=[];AVALI=[];
    for j=1:length(evec)
        ZR=[ZR,real(evec(:,j))];
        ZI=[ZI,imag(evec(:,j))];
        AVALR=[AVALR;real(eval(j,j))];
        AVALI=[AVALI;imag(eval(j,j))];
    end

    ZZu=[];lambdau=[];
    contr=0.0;
for j=1:length(evec)
    if abs(AVALI(j))>0.000001
       if abs((AVALR(j)^2+AVALI(j)^2)^(0.5))>=1
                         lambdau=[lambdau; (AVALR(j)^2+AVALI(j)^2)^(0.5)];
	     if contr~=abs((AVALR(j)^2+AVALI(j)^2)^(0.5))
                         ZZu=[ZZu ZR(:,j) ZI(:,j)];
	     end
	     contr=abs((AVALR(j)^2+AVALI(j)^2)^(0.5));             
       end
       elseif abs(AVALR(j))>=1
           lambdau=[lambdau; AVALR(j)];
           ZZu=[ZZu,ZR(:,j)];
    end
end

    ZZs=[];lambdas=[];
    contr=0.0;
for j=1:length(evec)
    if abs(AVALI(j))>0.000001
       if abs((AVALR(j)^2+AVALI(j)^2)^(0.5))<1
                         lambdas=[lambdas; (AVALR(j)^2+AVALI(j)^2)^(0.5)];
	     if contr~=abs((AVALR(j)^2+AVALI(j)^2)^(0.5))
                         ZZs=[ZZs ZR(:,j) ZI(:,j)];
                         
	     end
	     contr=abs((AVALR(j)^2+AVALI(j)^2)^(0.5));             
       end
       elseif abs(AVALR(j))<1
           lambdas=[lambdas; AVALR(j)];
           ZZs=[ZZs,ZR(:,j)];
    end
end
  vlambda=[lambdas; lambdau];
  Mlambda=diag(vlambda);
  ZZ=[ZZs ZZu];
  
  Mm=ZZ';
    
% Computing matrix CONT1, CONT2, MS1, MS2

MCONT1=-inv(Mm(4:5,1:2))*Mm(4:5,3:5);
MCONT2=-inv(Mm(4:5,:)*MGamma4)*Mm(4:5,:)*MGamma5;

MS1=MGamma3(3:5,1:2)*MCONT1+MGamma3(3:5,3:5);
MS2=MGamma5(3:5,:)+MGamma4(3:5,:)*MCONT2;


% Computing paths for state and control variables

state=[];

control=[];

stateant=[log(k0/kss); log(A0/Ass); log((1+x0)/(1+xss))];

state=[state stateant];

for t=1:nobs
    state1=MS1*stateant+MS2*eps(:,t);
    
    controlt=MCONT1*stateant;
    
    stateant=state1;
    
    state=[state stateant];
    
    control=[control controlt];
        
end

state=state';
ktilde=log(kss)+state(:,1);
k=exp(ktilde);
Atilde=log(Ass)+state(:,2);
A=exp(Atilde);
xtilde=log(1+xss)+state(:,3);
x=exp(xtilde)-1;

control=control';
ctilde=log(css)+control(:,1);
c=exp(ctilde);
mtilde=log(mss)+control(:,2);
m=exp(mtilde);


y=A.*(k.^(alpha));             % y(t)

nr=(1./(1-theta.*(c./m)))-1;   % i(t)

xc=xtilde-ones(length(xtilde),1)*log(1+xss);
Ac=Atilde-ones(length(Atilde),1)*log(Ass);
kc=ktilde-ones(length(ktilde),1)*log(kss);
rtilde=log(1+rss)+(1/(1+rss))*(1-tauy)*alpha*Ass*(kss^(alpha-1)).*(rhoA.*Ac+(alpha-1).*kc);
r=exp(rtilde)-1;               % r(t)


% Computing paths for M(t), P(t)
M=[];
P=[];
Mant=M0;
M=[M; Mant];
P0=Mant/m(1); Pant=P0;
P=[P; Pant];

for t=1:nobs-1
    M1=(1+x(t+1))*Mant;
    P1=M1/m(t+1);
    Mant=M1;
    M=[M;Mant];
    P=[P;P1];
end

% Computing pi(t)

pitilde=xtilde(1:nobs-1)+mtilde(1:nobs-1)-mtilde(2:nobs);
pit=exp(pitilde)-1;
pitet=xtilde(1:nobs-1)+mtilde(1:nobs-1)-log(mss)-MCONT1(2,1).*kc(1:nobs-1)-MCONT1(2,2).*Ac(1:nobs-1)-MCONT1(2,3).*xc(1:nobs-1);
pite=exp(pitet)-1;


% Computing real debt path
eta=rss+0.1;
trss=0.4*yss;
b=[];b=[b; b0];
btant=b0;
for t=1:nobs-1
    bt1=(1+r(t))*(1-eta)*btant+(trss-tauy*y(t)-tauc*c(t)-((M(t+1)-M(t))/P(t)))*(1+nr(t));
    btant=bt1;
    b=[b; btant];
end
bss=(1/((1-eta)*(1+rss)))*(trss-tauy*yss-tauc*css-piss*mss/(1+piss));

T=length(pit);
ferror1=-(c(1:T-1).^(-sigma)).*(m(1:T-1).^(theta*(1-sigma)))+...
    beta.*(c(2:T).^(-sigma)).*(m(2:T).^(theta*(1-sigma))).*((1-tauy)*...
    alpha.*A(2:T).*(k(2:T).^(alpha-1))+1-delta);
ferror2=-(c(1:T-1).^(-sigma)).*(m(1:T-1).^(theta*(1-sigma)))+...
    beta.*(c(2:T).^(-sigma)).*(m(2:T).^(theta*(1-sigma))).*((1+nr(1:T-1))./(1+pit(2:T)));
