function [y,c,k,m,A,x,nr,r,pit,pite,ferror1,ferror2]=S_i_sr(eps,X0,XSS,nobs,param)
% Copyright J. Ruiz (2006)


% structural parameters
alpha=param(1);     % Output elasticity of capital stock
delta=param(2);     % Depreciation rate of capital
theta=param(3);     % weight of the real money in utility
sigma=param(4);     % Inverse of intertemporal elasticity of consumption
beta=param(5);      % Time discount factor
tauy=param(6);      % Income tax rate
tauc=param(7);      % Consumption tax rate
iss=param(8);       % growth rate of money supply
rhoi=param(9);      % autoregressive parameter for the growth rate of money supply
Ass=param(10);      % steady state for technology variable
rhoA=param(11);     % Autoregressive parameter for the technology variable
rhoy=param(12);     % output coefficient in Taylor rule
rhopi=param(13);    % inflation coefficient in Taylor rule

% Steady state

kss=XSS(1);
css=XSS(2);
rss=XSS(3);
piss=XSS(4);
xss=XSS(5);
mss=XSS(6);
yss=XSS(7);

% initial conditions

k0=X0(1);
b0=X0(2);
M0=X0(3);
i0=X0(4);
A0=X0(5);

% Computing Matrix B

MB=[0 rhopi rhoy*alpha rhoi];

% Computing Matrix D, J, F, G, H, L, N, Q

MD=[(sigma-theta*(1-sigma))/beta  0       (1-alpha)*((1/beta)-(1-delta))  -theta*(1-sigma)/(beta*iss);
    (sigma-theta*(1-sigma))/beta  1/beta  0                               -(theta*(1-sigma)/(beta*iss)+((1+iss)/(1+piss)));
    0                             0       kss                             0;
    0                             0       0                               1];

MJ=[((1/beta)-(1-delta))*rhoA;
    0;
    Ass*(kss^alpha);
    0];

MF=[theta*(1-sigma)/(beta*iss);
    theta*(1-sigma)/(beta*iss);
    0;
    0];

MG=[(sigma-theta*(1-sigma))/beta   0  0                                     0;
    (sigma-theta*(1-sigma))/beta   0  0                                     0;
    -css                           0  alpha*Ass*(kss^alpha)+(1-delta)*kss   0;
    0                              0  0                                     0];

MH=[0;
    0;
    0;
    1];

ML=MD+MF*MB;
MN=MG+MH*MB;
MQ=MJ+rhoA.*MH-(rhoA*rhoy).*MF;

% Computing Matrix Gamma1, Gamma2, Gamma3

MGamma1=inv(ML)*MN;
MGamma2=inv(ML)*MQ;
MGamma3=inv(ML)*MH;

% Computing matrix M^(-1),  L, Q

  [evec,eval]=eig(MGamma1);
  
  eval2=diag(eval);

  [eval3,nord]=sort(abs(eval2));

  eval4=eval2(nord);

  Mlambda=diag(eval4);

  MM=evec(:,nord);

  Mm=inv(MM);

  MPHI=Mm*MGamma2;
  
  MPSI=Mm*MGamma3;

  
% Computing matrix S, U, V, W, THETA, XI, OMEGA, CONT1 CONT2

MS=[Mm(3,1) Mm(3,2); Mm(4,1)  Mm(4,2)];
MU=[-Mm(3,3) -Mm(3,4); -Mm(4,3) -Mm(4,4)];
MV=[MPHI(3)/(rhoA-Mlambda(3,3));
    MPHI(4)/(rhoA-Mlambda(4,4))];
MW=[-MPSI(3)/Mlambda(3,3);
    -MPSI(4)/Mlambda(4,4)];
MTHETA=inv(MS)*MU;
MXI=inv(MS)*MV;
MOMEGA=inv(MS)*MW;

MCONT1=[MTHETA  rhoA.*MXI];
MCONT2=[MOMEGA MXI];

% Computing matrix Bt Dt Ft Gt Ht Jt Lt Qt Ut Vt S1 S2

MBt=[Mm(1,1) Mm(1,2); Mm(2,1) Mm(2,2)];
MDt=[Mm(1,3) Mm(1,4); Mm(2,3) Mm(2,4)];
MFt=[0 0; Mlambda(2,2)*Mm(2,1)  Mlambda(2,2)*Mm(2,2)];
MGt=[0 0; Mlambda(2,2)*Mm(2,3)  Mlambda(2,2)*Mm(2,4)];
MHt=[MPHI(1); MPHI(2)];
MJt=[MPSI(1); MPSI(2)];
MLt=MBt*MTHETA+MDt;
MQt=MFt*MTHETA+MGt;
MUt=MHt+(MFt-rhoA.*MBt)*MXI;
MVt=MJt+MFt*MOMEGA;

MS1=[inv(MLt)*MQt   rhoA.*inv(MLt)*MUt; zeros(1,2)  rhoA];

MS2=[inv(MLt)*MVt  inv(MLt)*MUt; 0  1];



% Computing paths for state and control variables

state=[];

control=[];

stateant=[log(k0/kss); log((1+i0)/(1+iss)); log(A0/Ass)];

state=[state stateant];

for t=1:nobs
    state1=MS1*stateant+MS2*eps(:,t);
    
    controlt=MCONT1*stateant+MCONT2*eps(:,t);
    
    stateant=state1;
    
    state=[state stateant];
    
    control=[control controlt];
        
end

state=state';
ktilde=log(kss)+state(:,1);
k=exp(ktilde);
Atilde=log(Ass)+state(:,3);
A=exp(Atilde);
nrtilde=log(1+iss)+state(:,2);
nr=exp(nrtilde)-1;

control=control';
ctilde=log(css)+control(:,1);
c=exp(ctilde);
pitilde=log(1+piss)+control(:,2);
pit=exp(pitilde)-1;


m=theta.*((1+nr(1:length(c)))./nr(1:length(c))).*c;  % m(t)

y=A.*(k.^(alpha));  % y(t)

Ac=Atilde-ones(length(Atilde),1)*log(Ass);
kc=ktilde-ones(length(ktilde),1)*log(kss);
rtilde=log(1+rss)+(1/(1+rss))*(1-tauy)*alpha*Ass*(kss^(alpha-1)).*(rhoA.*Ac+(alpha-1).*kc);
r=exp(rtilde)-1;   % r(t)

pietilde=nrtilde(2:nobs)-rtilde(2:nobs);   % E(pitilde(t+1))
pite=pietilde;

x=(m(2:length(c))./m(1:length(c)-1)).*(1+pit(2:length(c)))-1;

T=length(pit);
ferror1=-(c(1:T-1).^(-sigma)).*(m(1:T-1).^(theta*(1-sigma)))+...
    beta.*(c(2:T).^(-sigma)).*(m(2:T).^(theta*(1-sigma))).*((1-tauy)*...
    alpha.*A(2:T).*(k(2:T).^(alpha-1))+1-delta);
ferror2=-(c(1:T-1).^(-sigma)).*(m(1:T-1).^(theta*(1-sigma)))+...
    beta.*(c(2:T).^(-sigma)).*(m(2:T).^(theta*(1-sigma))).*((1+nr(1:T-1))./(1+pit(2:T)));


