function [yt,ct,kt,mt,it,pit,piet,invt,qt,wt,dt,zt,at,et,vt,ferror1,ferror2,ferror3]=nkeyn(eps,X0,XSS,nobs,param)

% structural parameters

alpha=param(1);        % Output elasticity of capital stock
delta=param(2);        % Depreciation rate of capital
theta=param(3);        % weight of the real money in utility
sigma=param(4);        % Inverse of intertemporal elasticity of consumption
beta=param(5);         % Time discount factor
taur=param(6);         % Capital Income tax rate
tauw=param(7);         % Labor Income tax rate
tauc=param(8);         % Consumption tax rate
phi=param(9);          % cost of adjusting nominal price parameter
epsi=param(10);        % parameter of the technology of finished good-producing firm
rhoi=param(11);        % autoregressive parameter for nominal return rule
Ass=param(12);         % steady state for technology variable
rhoz=param(13);        % Autoregressive parameter for the technology variable
rhoy=param(14);        % parameter Taylor rule
rhopi=param(15);       % parameter Taylor rule
rhoa=param(16);        % autoregressive parameter for the preference shock
rhoe=param(17);        % autoregressive parameter for the shock to money demand
rhov=param(18);        % autoregressive parameter for the shock to the marginal efficiency of investment

% Steady state
iss=XSS(1);       % nominal return that the monetary authority chooses

qss=XSS(2);
piss=XSS(3);
kss=XSS(4);
css=XSS(5);
mss=XSS(6);
yss=XSS(7);
wss=XSS(8);
dss=XSS(9);
invss=XSS(10);

zss=XSS(11);
ass=XSS(12);
ess=XSS(13);
vss=XSS(14);

% initial conditions
b0=X0(1);
M0=X0(2);
k0=X0(3);
i0=X0(4);
z0=X0(5);
a0=X0(6);
e0=X0(7);
v0=X0(8);


% Computing Matrix D, G, H, J

MD=[(sigma-theta*(1-sigma))/beta  theta*(1-sigma)*rhopi/(beta*iss)            -(1-taur)  theta*(1-sigma)*rhoy*alpha/(beta*iss)  -theta*(1-sigma)*(1-rhoi)/(beta*iss);
    (sigma-theta*(1-sigma))/beta  (1/beta)+theta*(1-sigma)*rhopi/(beta*iss)   0          theta*(1-sigma)*rhoy*alpha/(beta*iss)  -(theta*(1-sigma)*(1-rhoi)/(beta*iss))-1/(1+piss);
    0                             beta*phi                                    0          0                                       0;
    0                             0                                           0          1                                       0;
    0                             0                                           0          0                                       1];

MG=[(sigma-theta*(1-sigma))/beta  0      0       0                                 0;
    (sigma-theta*(1-sigma))/beta  0      0       0                                 0;
    0                             phi    1-epsi  (1-alpha)*(1-epsi)                0;
    -css/kss                      0      0       alpha*Ass*kss^(alpha-1)+1-delta   0;
    0                             rhopi  0       rhoy*alpha                        rhoi];

MH=[-theta*(1-sigma)*rhoy*rhoz*(1-alpha)/(beta*iss)   -(1-rhoa)/beta   theta*(1-sigma)*(1-rhoe)/beta  (1-rhov)/beta;
    0                                                 0               0                               (1-rhov)/beta;
    (alpha-1)*(epsi-1)                                0               0                               0;
    (1-alpha)*Ass*kss^(alpha-1)                       0               0                               delta;
    rhoy*(1-alpha)                                    0               0                               0];

MJ=[0;0;0;0;1];


% Computing Matrix Gamma1, Gamma2, Gamma3, Gamma4, Gamma5, Gamma6

MGamma1=inv(MD)*MG;
MGamma2=inv(MD)*MH;
MGamma3=inv(MD)*MJ;

Mzeros45=zeros(4,5);
MOmega=[rhoz 0    0    0;
        0    rhoa 0    0;
        0    0    rhoe 0;
        0    0    0    rhov];
MGamma4=[MGamma1 MGamma2; Mzeros45 MOmega];
Mzeros63=zeros(6,3);
MGamma5=[eye(3);Mzeros63];
Mzeros54=zeros(5,4);
Mzeros41=zeros(4,1);
MGamma6=[Mzeros54 MGamma3; eye(4) Mzeros41];

% Computing matrix M Mtilde mtilde Psi1 Psi2 Psi3 Psi4

  [evec,eval]=eig(MGamma4);
  
  eval2=diag(eval);

  [eval3,nord]=sort(abs(eval2));

  eval4=eval2(nord);

  Mlambda=diag(eval4);

  MM=evec(:,nord);

  MMtilde=[real(MM(:,1:7)) imag(MM(:,8)) MM(:,9)];
  
  Mmtilde=inv(MMtilde);
  
  MPsi1=-inv(Mmtilde(7:9,1:3))*Mmtilde(7:9,4:9);
  
  MPsi2=-inv(Mmtilde(7:9,:)*MGamma5)*(Mmtilde(7:9,:)*MGamma6);
  
  MPsi3=MGamma4(4:9,1:3)*MPsi1+MGamma4(4:9,4:9);
  
  MPsi4=MGamma6(4:9,:);
  
% Computing paths for state and control variables

state=[];

control=[];

stateant=[log(k0/kss); log(i0/iss); log(z0/zss); log(a0/ass); log(e0/ess); log(v0/vss)];

state=[state stateant];

for t=1:nobs
    state1=MPsi3*stateant+MPsi4*eps(:,t);
    control1=MPsi1*stateant;
    stateant=state1;
    state=[state stateant];
    control=[control control1];
end

%Computing capital, nominal interest rate and shocks (z,a,e,v)
state=state';
ktilde=log(kss)+state(:,1);
kt=exp(ktilde);
itilde=log(1+iss)+state(:,2);  %itilde=log(1+i(t))
it=exp(itilde)-1;
ztilde=log(zss)+state(:,3);
zt=exp(ztilde);
atilde=log(ass)+state(:,4);
at=exp(atilde);
etilde=log(ess)+state(:,5);
et=exp(etilde);
vtilde=log(vss)+state(:,6);
vt=exp(vtilde);

%Computing consumption, inflation, and real rental rate for capital
control=control';
ctilde=log(css)+control(:,1);
ct=exp(ctilde);
pitilde=log(1+piss)+control(:,2);
pit=exp(pitilde)-1;
qtilde=log(qss)+control(:,3);
qt=exp(qtilde);

%Computing output, real money, wages, dividends and investment
yt=Ass*zt.^(1-alpha).*kt.^alpha;
mt=theta*(1+tauc)*((1+it(1:length(ct)))./it(1:length(ct))).*ct;
wt=((1-alpha)/alpha)*qt.*kt(1:length(qt));
dt=yt(1:length(qt))-wt-qt.*kt(1:length(qt))-(phi/2)*yt(1:length(qt)).*(((1+pit)/(1+piss))-1).^2;
invt=(kt(2:length(kt))-(1-delta)*kt(1:length(qt)))./vt(1:length(qt));

% Computing expected inflation: E[log(1+pit(t+1))|I(t)]
piet=log(1+piss)+(1/(beta*phi))*(phi*control(:,2)-(epsi-1)*control(:,3)-...
    (1-alpha)*(epsi-1)*state(1:length(qt),1)+(1-alpha)*(epsi-1)*state(1:length(qt),3));

T=length(ct);

% Computing expectation errors

ferror1=-(at(1:T-1)./vt(1:T-1)).*(ct(1:T-1).^(-sigma)).*(mt(1:T-1).^(theta*(1-sigma))).*(et(1:T-1).^(-theta*(1-sigma)))+...
    beta.*at(2:T).*(ct(2:T).^(-sigma)).*(mt(2:T).^(theta*(1-sigma))).*(et(2:T).^(-theta*(1-sigma))).*((1-taur).*qt(2:T)+(1-delta)./vt(2:T));

ferror2=-at(1:T-1).*(ct(1:T-1).^(-sigma)).*(mt(1:T-1).^(theta*(1-sigma))).*(et(1:T-1).^(-theta*(1-sigma)))+...
    beta.*(1+it(1:T-1)).*at(2:T).*(ct(2:T).^(-sigma)).*(mt(2:T).^(theta*(1-sigma))).*(et(2:T).^(-theta*(1-sigma)))./(1+pit(2:T));

lambdat=at(1:T).*(ct(1:T).^(-sigma)).*(mt(1:T).^(theta*(1-sigma))).*(et(1:T).^(-theta*(1-sigma)));
ferror3=-phi*(((1+pit(1:T-1))/(1+piss))-1).*((1+pit(1:T-1))/(1+piss))+...
    (1-epsi)+epsi*qt(1:T-1).*kt(1:T-1)./(alpha*yt(1:T-1))+...
    beta*phi*(lambdat(2:T)./lambdat(1:T-1)).*(((1+pit(2:T))/(1+piss))-1).*((1+pit(2:T))/(1+piss)).*(yt(2:T)./yt(1:T-1));

